@@ -49,9 +49,13 @@ void NLOPT_STDCALL nlopt_destroy(nlopt_opt opt)
             free(opt->fc[i].tol);
         for (i = 0; i < opt->p; ++i)
             free(opt->h[i].tol);
+        for (i = 0; i < opt->nparams; ++i)
+            free(opt->params[i].name);
+        free(opt->params);
         free(opt->lb);
         free(opt->ub);
         free(opt->xtol_abs);
+        free(opt->x_weights);
         free(opt->fc);
         free(opt->h);
         nlopt_destroy(opt->local_opt);
@@ -84,10 +88,13 @@ nlopt_opt NLOPT_STDCALL nlopt_create(nlopt_algorithm algorithm, unsigned n)
         opt->fc = NULL;
         opt->p = opt->p_alloc = 0;
         opt->h = NULL;
+        opt->params = NULL;
+        opt->nparams = 0;
 
         opt->stopval = -HUGE_VAL;
         opt->ftol_rel = opt->ftol_abs = 0;
         opt->xtol_rel = 0;
+        opt->x_weights = NULL;
         opt->xtol_abs = NULL;
         opt->maxeval = 0;
         opt->numevals = 0;
@@ -133,7 +140,7 @@ nlopt_opt NLOPT_STDCALL nlopt_copy(const nlopt_opt opt)
         nlopt_munge munge;
         nopt = (nlopt_opt) malloc(sizeof(struct nlopt_opt_s));
         *nopt = *opt;
-        nopt->lb = nopt->ub = nopt->xtol_abs = NULL;
+        nopt->lb = nopt->ub = nopt->xtol_abs = nopt->x_weights = NULL;
         nopt->fc = nopt->h = NULL;
         nopt->m_alloc = nopt->p_alloc = 0;
         nopt->local_opt = NULL;
@@ -141,6 +148,8 @@ nlopt_opt NLOPT_STDCALL nlopt_copy(const nlopt_opt opt)
         nopt->work = NULL;
         nopt->errmsg = NULL;
         nopt->force_stop_child = NULL;
+        nopt->params = NULL;
+        nopt->nparams = 0;
 
         munge = nopt->munge_on_copy;
         if (munge && nopt->f_data)
@@ -157,6 +166,12 @@ nlopt_opt NLOPT_STDCALL nlopt_copy(const nlopt_opt opt)
             nopt->xtol_abs = (double *) malloc(sizeof(double) * (opt->n));
             if (!opt->xtol_abs)
                 goto oom;
+            if (opt->x_weights) {
+                nopt->x_weights = (double *) malloc(sizeof(double) * (opt->n));
+                if (!opt->x_weights)
+                    goto oom;
+                memcpy(nopt->x_weights, opt->x_weights, sizeof(double) * (opt->n));
+            }
 
             memcpy(nopt->lb, opt->lb, sizeof(double) * (opt->n));
             memcpy(nopt->ub, opt->ub, sizeof(double) * (opt->n));
@@ -209,6 +224,19 @@ nlopt_opt NLOPT_STDCALL nlopt_copy(const nlopt_opt opt)
                 }
         }
 
+        if (opt->nparams) {
+            nopt->nparams = opt->nparams;
+            nopt->params = (nlopt_opt_param *) calloc(opt->nparams, sizeof(nlopt_opt_param));
+            if (!nopt->params) goto oom;
+            for (i = 0; i < opt->nparams; ++i) {
+                size_t len = strlen(opt->params[i].name) + 1;
+                nopt->params[i].name = (char *) malloc(len);
+                if (!nopt->params[i].name) goto oom;
+                memcpy(nopt->params[i].name, opt->params[i].name, len);
+                nopt->params[i].val = opt->params[i].val;
+            }
+        }
+
         if (opt->local_opt) {
             nopt->local_opt = nlopt_copy(opt->local_opt);
             if (!nopt->local_opt)
@@ -230,6 +258,65 @@ nlopt_opt NLOPT_STDCALL nlopt_copy(const nlopt_opt opt)
     return NULL;
 }
 
+/*************************************************************************/
+/* generic algorithm parameters, implemented as a simple array of (name,val)
+   pairs that can interpreted as needed by individual algorithms.
+
+   (No point in a fancier data structure since only a handful of these
+   should be set in practice). */
+
+nlopt_result nlopt_set_param(nlopt_opt opt, const char *name, double val) {
+    size_t len;
+    unsigned i;
+    if (!opt) RETURN_ERR(NLOPT_INVALID_ARGS, opt, "invalid NULL opt");
+    if (!name) RETURN_ERR(NLOPT_INVALID_ARGS, opt, "invalid NULL parameter name");
+    len = strnlen(name, 1024) + 1;
+    if (len > 1024) RETURN_ERR(NLOPT_INVALID_ARGS, opt, "parameter name must be < 1024 bytes");
+    for (i = 0; i < opt->nparams; ++i)
+        if (!strcmp(name, opt->params[i].name))
+            break;
+    if (i == opt->nparams) { /* allocate new parameter */
+        opt->nparams++;
+        opt->params = (nlopt_opt_param *) realloc(opt->params, sizeof(nlopt_opt_param) * opt->nparams);
+        if (!opt->params) return NLOPT_OUT_OF_MEMORY;
+        opt->params[i].name = (char *) malloc(len);
+        if (!opt->params[i].name) return NLOPT_OUT_OF_MEMORY;
+        memcpy(opt->params[i].name, name, len);
+    }
+    opt->params[i].val = val;
+    return NLOPT_SUCCESS;
+}
+
+double nlopt_get_param(const nlopt_opt opt, const char *name, double defaultval)
+{
+    unsigned i;
+    if (!opt || !name || strnlen(name, 1024) == 1024) return defaultval;
+    for (i = 0; i < opt->nparams; ++i)
+        if (!strcmp(name, opt->params[i].name))
+            return opt->params[i].val;
+    return defaultval;
+}
+
+int nlopt_has_param(const nlopt_opt opt, const char *name)
+{
+    unsigned i;
+    if (!opt || !name || strnlen(name, 1024) == 1024) return 0;
+    for (i = 0; i < opt->nparams; ++i)
+        if (!strcmp(name, opt->params[i].name))
+            return 1;
+    return 0;
+}
+
+unsigned nlopt_num_params(const nlopt_opt opt)
+{
+    return opt ? opt->nparams : 0;
+}
+
+const char *nlopt_nth_param(const nlopt_opt opt, unsigned n)
+{
+    return opt && n < opt->nparams ? opt->params[n].name : NULL;
+}
+
 /*************************************************************************/
 
 nlopt_result NLOPT_STDCALL nlopt_set_precond_min_objective(nlopt_opt opt, nlopt_func f, nlopt_precond pre, void *f_data)
@@ -624,8 +711,64 @@ nlopt_result NLOPT_STDCALL nlopt_set_xtol_abs1(nlopt_opt opt, double xtol_abs)
 
 nlopt_result NLOPT_STDCALL nlopt_get_xtol_abs(const nlopt_opt opt, double *xtol_abs)
 {
-    memcpy(xtol_abs, opt->xtol_abs, opt->n * sizeof(double));
-    return NLOPT_SUCCESS;
+    nlopt_unset_errmsg(opt);
+    if (opt && (opt->n == 0 || xtol_abs)) {
+        memcpy(xtol_abs, opt->xtol_abs, opt->n * sizeof(double));
+        return NLOPT_SUCCESS;
+    }
+    return NLOPT_INVALID_ARGS;
+}
+
+nlopt_result NLOPT_STDCALL nlopt_set_x_weights(nlopt_opt opt, const double *x_weights)
+{
+    if (opt) {
+        unsigned i;
+        nlopt_unset_errmsg(opt);
+        for (i = 0; i < opt->n; i++)
+            if (x_weights[i] < 0)
+                return ERR(NLOPT_INVALID_ARGS, opt, "invalid negative weight");
+        if (!opt->x_weights && opt->n > 0) {
+            opt->x_weights = (double *) calloc(opt->n, sizeof(double));
+            if (!opt->x_weights) return NLOPT_OUT_OF_MEMORY;
+        }
+        if (opt->n > 0) memcpy(opt->x_weights, x_weights, opt->n * sizeof(double));
+        return NLOPT_SUCCESS;
+    }
+    return NLOPT_INVALID_ARGS;
+}
+
+nlopt_result NLOPT_STDCALL nlopt_set_x_weights1(nlopt_opt opt, double x_weight)
+{
+    if (opt) {
+        unsigned i;
+        if (x_weight < 0) return ERR(NLOPT_INVALID_ARGS, opt, "invalid negative weight");
+        nlopt_unset_errmsg(opt);
+        if (!opt->x_weights && opt->n > 0) {
+            opt->x_weights = (double *) calloc(opt->n, sizeof(double));
+            if (!opt->x_weights) return NLOPT_OUT_OF_MEMORY;
+        }
+        for (i = 0; i < opt->n; ++i)
+            opt->x_weights[i] = x_weight;
+        return NLOPT_SUCCESS;
+    }
+    return NLOPT_INVALID_ARGS;
+}
+
+nlopt_result NLOPT_STDCALL nlopt_get_x_weights(const nlopt_opt opt, double *x_weights)
+{
+    if (opt) {
+	if (opt->n > 0 && !x_weights) return ERR(NLOPT_INVALID_ARGS, opt, "invalid NULL weights");
+        nlopt_unset_errmsg(opt);
+        if (opt->x_weights) {
+            memcpy(x_weights, opt->x_weights, sizeof(double) * (opt->n));
+        } else {
+            unsigned i;
+            for (i = 0; i < opt->n; ++i)
+                x_weights[i] = 1;
+        }
+        return NLOPT_SUCCESS;
+    }
+    return NLOPT_INVALID_ARGS;
 }
 
 GETSET(maxeval, int, maxeval)
